//+------------------------------------------------------------------+
//|                                               CStrategyStoch.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#include <CSampleStrategy.mqh>
//+------------------------------------------------------------------+
//|  CStrategyStoch             |
//|             |
//+------------------------------------------------------------------+
class CStrategyStoch : public CSampleStrategy
  {
protected:
   int               m_handle;          //    Stochastic Oscillator (iStochastic)
   int               m_period_k;        // K- (   )
   int               m_period_d;        // D- (  )
   int               m_period_slowing;  //  
   double            m_main_line[];     //     
   double            m_signal_line[];   //     
public:
   //  
   int               Initialization(int period_k,int period_d,int period_slowing,bool virtual_trade_flag);
   //  
   int               Deinitialization();
   //       
   bool              CheckTradeConditions();
  };
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
int CStrategyStoch::Initialization(int period_k,int period_d,int period_slowing,bool virtual_trade_flag)
  {
//   
   m_period_k=period_k;
   m_period_d=period_d;
   m_period_slowing=period_slowing;

//     
   SetVirtualTradeFlag(virtual_trade_flag);

//     
   ArraySetAsSeries(m_rates,true);
   ArraySetAsSeries(m_main_line,true);
   ArraySetAsSeries(m_signal_line,true);

//   
   m_handle=iStochastic(_Symbol,_Period,m_period_k,m_period_d,m_period_slowing,MODE_SMA,STO_LOWHIGH);
   if(m_handle<0)
     {
      Alert("    Stochastic -  : ",GetLastError(),"!!");
      return(-1);
     }

   return(0);
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
int CStrategyStoch::Deinitialization()
  {
//   
   Position_CloseOpenedPosition(m_position);
//  
   IndicatorRelease(m_handle);
   return(0);
  };
//+------------------------------------------------------------------+
//|                |
//+------------------------------------------------------------------+
bool CStrategyStoch::CheckTradeConditions()
  {
//    
   RecalcPositionProperties(m_position);
   double p_close;

//---     3- 
   if(CopyRates(_Symbol,_Period,0,3,m_rates)<0)
     {
      Alert("    - :",GetLastError(),"!!");
      return(false);
     }
//       (  1)
   p_close=m_rates[1].close;  //              

   if((CopyBuffer(m_handle,0,0,3,m_main_line)<3) || (CopyBuffer(m_handle,1,0,3,m_signal_line)<3))
     {
      Alert("    Stochastic -  :",GetLastError());
      return(false);
     }

   //  :      
   bool buy_condition=((m_signal_line[2]<m_main_line[2]) && (m_signal_line[1]>m_main_line[1]));
   //  :      
   bool sell_condition=((m_signal_line[2]>m_main_line[2]) && (m_signal_line[1]<m_main_line[1]));

   int new_state=0;

   if(buy_condition) new_state=SIGNAL_OPEN_LONG;
   if(sell_condition) new_state=SIGNAL_OPEN_SHORT;

   if((GetSignalState()==SIGNAL_OPEN_SHORT) && (buy_condition)) new_state=SIGNAL_CLOSE_SHORT;
   if((GetSignalState()==SIGNAL_OPEN_LONG) && (sell_condition)) new_state=SIGNAL_CLOSE_LONG;

   if(GetSignalState()!=new_state)
     {
      SetSignalState(new_state);
     }

   return(true);
  };
//+------------------------------------------------------------------+